# -*- coding: UTF-8 -*-
__author__ = "yuyangit"
"""
  * @File    :   File.py
  * @Time    :   2023/04/22 19:25:26
  * @Author  :   余洋 
  * @Version :   1.0
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   None
"""
import os
import re
import logging
from pathlib import Path
import shutil


from .Filter import FileTypeFilter


class File:
    @staticmethod
    def clean(
        keyword: str | None,
        cwd: Path | str | None,
        glob_keyword: str | None,
        file_type_filter: int | str = FileTypeFilter.ANY.value,
        verbose: bool = False,
        auto_cwd: bool = True,
        ignore_hidden_files: bool = True,
        should_delete_function=None,
        log_info=print,
        log_error=print,
        log_success=print,
    ):
        if not log_info or not callable(log_info):
            log_info = print
        if not cwd:
            if auto_cwd:
                cwd = Path.cwd().resolve()
            else:
                raise FileNotFoundError("请传入工作目录路径，或者设置为默认当前所在目录路径为工作目录")
        if not os.access(cwd, os.W_OK):
            raise PermissionError(f"当前目录({cwd})没有写入权限")
        if not keyword and not glob_keyword:
            raise ValueError("请传入关键字(-g/-k)进行过滤, -k支持正则表达式, 使用正则表达式搜索功能")
        if isinstance(cwd, str):
            cwd = Path(cwd)
        if not isinstance(file_type_filter, FileTypeFilter):
            file_type_filter = FileTypeFilter.parse(file_type_filter)
        file_path_list = []
        invalidate_file_path_list = []
        work_glob = cwd.rglob(glob_keyword) if isinstance(glob_keyword, str) else []
        for file_path in work_glob:
            matched = False
            if keyword:
                try:
                    match_object = re.search(
                        r"{keyword}".format(keyword=keyword), str(file_path), re.M
                    )
                    matched = match_object != None
                except Exception as exception:
                    raise ValueError(f"正则表达式匹配失败 => {exception}")
            else:
                matched = True
            if matched:
                ok = file_type_filter.validate(file_path)
                if ignore_hidden_files:
                    hidden_file_part = [
                        part for part in file_path.parts if part.startswith(".")
                    ]
                    ok = not hidden_file_part or len(hidden_file_part) == 0
                if ok and file_path.exists():
                    file_path_list.append(str(file_path))
                else:
                    invalidate_file_path_list.append(str(file_path))
        if file_path_list:
            for a_file_path in file_path_list:
                a_file_path = Path(a_file_path)
                file_type = "文件"
                if a_file_path.is_dir():
                    file_type = "目录"
                elif a_file_path.is_symlink():
                    file_type = "软链接"
                elif a_file_path.stat().st_nlink == 2:
                    file_type = "硬连接"
                elif a_file_path.is_socket():
                    file_type = "Socket"
                elif a_file_path.is_char_device():
                    file_type = "字符设备(char_device)"
                elif a_file_path.is_block_device():
                    file_type = "块设备(block_device)"
                elif a_file_path.is_fifo():
                    file_type = "管道"
                else:
                    file_type = "文件"
                msg = f"将要删除... => [{file_type}] {a_file_path}"
                log_info(msg)
            if (
                should_delete_function
                and callable(should_delete_function)
                and should_delete_function(file_path_list, invalidate_file_path_list)
            ):
                log_info("删除开始...")
                if not verbose:
                    log_info("正在删除...")
                success_file_path_list = []
                fail_file_path_list = []
                for file_path in file_path_list:
                    a_file_path = Path(file_path)
                    try:
                        if a_file_path.exists():
                            if verbose:
                                file_type = "文件"
                                if a_file_path.is_dir():
                                    file_type = "目录"
                                elif a_file_path.is_symlink():
                                    file_type = "软链接"
                                elif a_file_path.stat().st_nlink == 2:
                                    file_type = "硬连接"
                                elif a_file_path.is_socket():
                                    file_type = "Socket"
                                elif a_file_path.is_char_device():
                                    file_type = "字符设备(char_device)"
                                elif a_file_path.is_block_device():
                                    file_type = "块设备(block_device)"
                                elif a_file_path.is_fifo():
                                    file_type = "管道"
                                else:
                                    file_type = "文件"
                                log_info(f"正在删除... => [{file_type}] {file_path} ")
                            if a_file_path.is_dir():
                                shutil.rmtree(a_file_path)
                            else:
                                a_file_path.unlink()
                            success_file_path_list.append(str(a_file_path))
                    except:
                        fail_file_path_list.append(str(a_file_path))
                if not fail_file_path_list:
                    log_success("全部删除完成 !!!")
                else:
                    for a_file_path in fail_file_path_list:
                        a_file_path = Path(a_file_path)
                        if not a_file_path.exists():
                            continue
                        file_type = "文件"
                        if a_file_path.is_dir():
                            file_type = "目录"
                        elif a_file_path.is_symlink():
                            file_type = "软链接"
                        elif a_file_path.stat().st_nlink == 2:
                            file_type = "硬连接"
                        elif a_file_path.is_socket():
                            file_type = "Socket"
                        elif a_file_path.is_char_device():
                            file_type = "字符设备(char_device)"
                        elif a_file_path.is_block_device():
                            file_type = "块设备(block_device)"
                        elif a_file_path.is_fifo():
                            file_type = "管道"
                        else:
                            file_type = "文件"
                        msg = f"还未删除... => [{file_type}] {a_file_path}"
                        log_info(msg)
                    log_success(
                        f"已成功删除 ({len(file_path_list) - len(fail_file_path_list)}) 个文件或目录!!!"
                    )
                    log_info("还剩下以上文件或目录未删除: ==>")
            else:
                log_error("取消删除...")
        else:
            log_error("当前目录下暂无符合规则的文件或目录待清理")

    @staticmethod
    def touch(file_path: Path | str | None) -> Path | None:
        if not file_path:
            return None

        if isinstance(file_path, str):
            file_path = Path(file_path)

        if file_path.exists():
            if file_path.exists():
                return file_path
            return None

        try:
            file_path.parent.mkdir(parents=True, exist_ok=True)
            file_path.touch()
            return file_path.resolve()
        except Exception as exception:
            logging.exception(exception)
            return None



# -*- coding: UTF-8 -*-
__author__ = "yuyangit"
__doc__ = "Filter"
"""
  * @File    :   Filter.py
  * @Time    :   2023/04/23 21:53:04
  * @Author  :   余洋
  * @Version :   1.0
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   None
"""

from enum import IntEnum
from pathlib import Path


class FileTypeFilter(IntEnum):
    ANY = 0
    FILE = 1
    DIR = 2
    SYMLINK = 3
    HARDLINK = 4
    SOCKET = 5
    CHAR_DEVICE = 6
    BLOCK_DEVICE = 7
    FIFO = 8

    @classmethod
    def parse(cls, file_type_filter: str | int | None):
        if not file_type_filter:
            return FileTypeFilter.ANY

        names = [member for member in cls if member.name.upper() == file_type_filter]
        if not names and len(names) > 0:
            return names[0]

        values = [member for member in cls if member.value == file_type_filter]
        if not values and len(values) > 0:
            return values[0]

        return FileTypeFilter.ANY

    def validate(self, path: Path) -> bool:
        if not path or not path.exists():
            return False

        match self:
            case FileTypeFilter.ANY:
                return True
            case FileTypeFilter.FILE:
                return path.is_file()
            case FileTypeFilter.DIR:
                return path.is_dir()
            case FileTypeFilter.SYMLINK:
                return path.is_symlink()
            case FileTypeFilter.HARDLINK:
                return path.stat().st_nlink == 2
            case FileTypeFilter.SOCKET:
                return path.is_socket()
            case FileTypeFilter.CHAR_DEVICE:
                return path.is_char_device()
            case FileTypeFilter.BLOCK_DEVICE:
                return path.is_block_device()
            case FileTypeFilter.FIFO:
                return path.is_fifo()
            case _:
                return True



# -*- coding: UTF-8 -*-
__author__ = "yuyangit"
__doc__ = "Main"
"""
  * @File    :   Main.py
  * @Time    :   2023/06/10 14:06:17
  * @Author  :   余洋
  * @Version :   1.0
  * @Contact :   yuyangit.0515@qq.com
  * @License :   (C)Copyright 2019-2024, 希洋 (Ship of Ocean)
  * @Desc    :   None
"""

import argparse
import xy_file
from xy_file.File import File
import pkg_resources
from xy_file.utils import print_e, print_s, inputt, print_r


def should_delete(
    validate_file_path_list: list[str],
) -> bool:
    command = inputt(
        f"是否删除以上共 ({len(validate_file_path_list)}) 个文件和目录，\n\n【确定】请输入[Y], 输入其他按键进行【取消】。\n\n输入[Y/n] : \n"
    )
    return command == "Y"


def main():
    desc = f"{xy_file.__name__}-{pkg_resources.get_distribution(xy_file.__name__).version}: => 文件操作程序 - v{pkg_resources.get_distribution(xy_file.__name__).version}"
    print(desc)
    parser = argparse.ArgumentParser(
        prog=xy_file.__name__,
        description=desc,
    )
    work_list = [
        "clean",  # 必须添加-n参数作为新的project_name项目名称
    ]
    parser.add_argument(
        "-w",
        "--work",
        type=str,
        help="""
            命令:
            -----clean => 清理文件或目录
        """,
        choices=work_list,
        required=True,
    )
    parser.add_argument(
        "-g",
        "--glob-keyword",
        type=str,
        help="""
        工作目录过滤匹配关键字; 
        默认: **/*, 当前目录下的全部文件和目录，
        可选，工作目录过滤关键字，使用普通命令行文件匹配，例如 *.py表示当前目录下所有py脚本""",
        required=False,
        default="**/*",
    )
    parser.add_argument(
        "-k",
        "--keyword",
        type=str,
        help="关键字; 可选; 使用正则表达式Search; re.M模式, !!! 注意: 该关键字匹配是绝对路径字符串",
        required=False,
    )
    parser.add_argument(
        "-c",
        "--cwd",
        type=str,
        help="""工作目录; 默认命令行当前目录; 当auto_cwd为False且未传入工作目录时报错""",
        required=False,
    )
    parser.add_argument(
        "-i",
        "--ignore_hidden_files",
        type=int,
        help="""是否忽略隐藏目录和文件""",
        required=False,
        default=1,
    )
    parser.add_argument(
        "-f",
        "--file-type-filter",
        help="""
                        
                        过滤器, 默认为0; 无限制,
                        !!! 若输入字符串则无大小写限制,
                        以下为过滤器对应类型:
                        任意 = [0, any],
                        文件 = [1, file],
                  目录/文件夹 = [2, dir],
                      软链接 = [3, symlink],
                      硬链接 = [4, hardlink],
                     Socket = [5, socket],
        字符设备(char device) = [6, char_device],
        块设备(block device) = [7, block_device],
                  管道(fifo) = [8, fifo],
                        
                        """,
        default=0,
        required=False,
    )
    parser.add_argument(
        "-vb",
        "--verbose",
        type=bool,
        help="是否显示工作详情; 默认为0或者False",
        required=False,
        default=False,
    )
    parser.add_argument(
        "-ac",
        "--auto-cwd",
        type=bool,
        help="是否自动设置工作目录为当前目录; 默认为True",
        default=True,
    )
    work = parser.parse_args().work
    match work:
        case "clean":
            glob_keyword = parser.parse_args().glob_keyword
            keyword = parser.parse_args().keyword
            cwd = parser.parse_args().cwd
            file_type_filter = parser.parse_args().file_type_filter
            verbose = parser.parse_args().verbose
            auto_cwd = parser.parse_args().auto_cwd
            ignore_hidden_files = parser.parse_args().ignore_hidden_files
            try:
                ignore_hidden_files = bool(ignore_hidden_files)
            except:
                ignore_hidden_files = True
            try:
                File.clean(
                    glob_keyword=glob_keyword,
                    keyword=keyword,
                    cwd=cwd,
                    file_type_filter=file_type_filter,
                    verbose=verbose,
                    auto_cwd=auto_cwd,
                    ignore_hidden_files=ignore_hidden_files,
                    should_delete_function=should_delete,
                    log_info=print_r,  # type: ignore
                    log_success=print_s,  # type: ignore
                    log_error=print_e,  # type: ignore
                )
            except Exception as exception:
                print_e(f"删除失败 => {str(exception)}")

        case _:
            print_e("请输入-w/--work参数 !!!")


if __name__ == "__main__":
    main()
